<?php
/*
 * Plugin Name: WooCommerce Papam Payment Gateway
 * Plugin URI: https://www.afonepaiement.com
 * Description: Papam payment gateway on your store.
 * Author: MEP
 * Author URI: 
 * Version: 1.1.2
 * Text Domain: papam
 * Domain Path: /languages
 *
 */

add_action('init', 'start_session', 1);

function start_session() {
	if (!session_id()) {
		session_start();
	}
	if(!array_key_exists('papamtoken', $_SESSION)){
		$_SESSION['papamtoken'] = "";
	}
	session_write_close();
}

/*
* This action hook registers our PHP class as a WooCommerce payment gateway
*/
add_filter( 'woocommerce_payment_gateways', 'papam_add_gateway_class' );
function papam_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_Papam_Gateway'; // your class name is here
	return $gateways;
}

/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action( 'plugins_loaded', 'papam_init_gateway_class' );


function papam_init_gateway_class() {

	class WC_Papam_Gateway extends WC_Payment_Gateway {

		/**
		 * Class constructor, more about it in Step 3
		 */
		public function __construct() {

			$this->id = 'papam'; // payment gateway plugin ID

			$this->has_fields = true; // in case you need a custom credit card form
			$this->method_title = __('Papam', 'papam');
			$this->method_description = __('Papam payment gateway by Afone Paiement', 'papam'); // will be displayed on the options page

			// gateways can support subscriptions, refunds, saved payment methods,
			$this->supports = array(
				'products'
			);

			// Method with all the options fields
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();
			$this->title = $this->get_option( 'title' );
			$this->description = $this->get_option( 'description' );
			$this->enabled = $this->get_option( 'enabled' );
			$this->testmode = 'yes' === $this->get_option( 'testmode' );
			$this->publishable_key = $this->testmode ? $this->get_option( 'test_publishable_key' ) : $this->get_option( 'publishable_key' );
			$this->private_key = $this->testmode ? $this->get_option( 'test_private_key' ) : $this->get_option( 'private_key' );
			$this->url = $this->testmode ? $this->get_option( 'test_url' ) : $this->get_option( 'production_url' );
			$this->debiturl = $this->url . '/rest/payment/tokenDebit';

			$this->amex = $this->get_option( 'amex' );

			if ( "no" === $this->amex ) {
				$this->icon = plugin_dir_url( __FILE__ ) . 'assets/images/credit-card-logo-visamc.png'; // URL of the icon that will be displayed on checkout page near your gateway name
			}else{
				$this->icon = plugin_dir_url( __FILE__ ) . 'assets/images/credit-card-logo-visamcamex.png';
			}


			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// We need custom JavaScript to obtain a token
			add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );

		}

		/**
		 * Plugin options, we deal with it in Step 3 too
		 */
		public function init_form_fields(){

			$this->form_fields = array(
				'enabled' => array(
					'title'       => __('Enable/Disable', 'papam'),
					'label'       => __('Enable Papam Gateway', 'papam'),
					'type'        => 'checkbox',
					'description' => __('Check this field for enable Papam Gateway', 'papam'),
					'default'     => 'no',
					'desc_tip'    => true,
				),
				'title' => array(
					'title'       => __('Title', 'papam'),
					'type'        => 'text',
					'description' => __('This controls the title which the user sees during checkout.', 'papam'),
					'default'     => __('Pay with Credit Card', 'papam'),
					'desc_tip'    => true,
				),

				'description' => array(
					'title'       => __('Description', 'papam'),
					'type'        => 'textarea',
					'description' => __('This controls the description which the user sees during checkout', 'papam'),
					'default'     => __('Pay with your credit card', 'papam'),
				),
				'amex' => array(
					'title'       => __('American Express', 'papam'),
					'label'       => __('Enable American Express', 'papam'),
					'type'        => 'checkbox',
					'description' => __('Check if you would like to accept also American Express. You may need to request activation from your sales contact person.', 'papam'),
					'default'     => 'no',
					'desc_tip'    => true,
				),
				'testmode' => array(
					'title'       => __('Mode', 'papam'),
					'label'       => __('Enable Test Mode', 'papam'),
					'type'        => 'select',
					'description' => __('Place the payment gateway in test mode or production mode.', 'papam'),
					'default'     => 'yes',
					'desc_tip'    => true,
					'options' => array(
						'yes' => __('Homologation', 'papam'),
						'no' => __('Production', 'papam'),
					)
				),
				'test_url' => array(
					'title'       => __('API URL in homologation', 'papam'),
					'type'        => 'text',
					'default'     => 'https://secure-homologation.comnpay.com:60000',
				),
				'test_publishable_key' => array(
					'title'       => __('TPE in homologation', 'papam'),
					'type'        => 'text'
				),
				'test_private_key' => array(
					'title'       => __('Private Key in homologation', 'papam'),
					'type'        => 'text',
				),
				'production_url' => array(
					'title'       => __('API URL in production', 'papam'),
					'type'        => 'text',
					'default'     => 'https://secure.comnpay.com:60000',
				),
				'publishable_key' => array(
					'title'       => __('TPE in production', 'papam'),
					'type'        => 'text'
				),
				'private_key' => array(
					'title'       => __('Private Key in production', 'papam'),
					'type'        => 'text',
				)
			);
		}



		public function payment_fields() {

			// ok, let's display some description before the payment form
			if ( $this->description ) {
				// you can instructions for test mode, I mean test card numbers etc.
				if ( $this->testmode ) {
					$this->description .= '<br /><br />' . __(' TEST MODE ENABLED. In test mode, you can use the card numbers listed in our documentation', 'papam');
					$this->description  = trim( $this->description );
				}
				// display the description with <p> tags etc.
				echo wpautop( wp_kses_post( $this->description ) );
			}

			// I will echo() the form, but you can close PHP tags and print it directly in HTML
			echo '<fieldset id="wc-' . esc_attr( $this->id ) . '-cc-form" class="wc-credit-card-form wc-payment-form" style="background:transparent;">';

			// Add this action hook if you want your custom payment gateway to support it
			do_action( 'woocommerce_credit_card_form_start', $this->id );


			// I recommend to use inique IDs, because other gateways could already use #ccNo, #expdate, #cvc
			echo '<p class="form-row form-row-wide"><label>'.__('Card Number', 'papam').' <span class="required ">*</span></label>
        	        <input id="papam_ccNo" type="text" autocomplete="off" oninput="comNpay.checkAndcreateToken()"  class="classCardPapam">
        		</p>
        		<p class="form-row form-row-first">
        			<label>'.__('Expiry Date', 'papam').' <span class="required">*</span></label>
        			<input id="papam_expdate" type="text" autocomplete="off" placeholder="MM/YY" oninput="comNpay.checkAndcreateToken()" class="classCardPapam">
        		</p>
        		<p class="form-row form-row-last">
        			<label>'.__('Card Code (CVV)', 'papam').' <span class="required"></span></label>
        			<input id="papam_cvv" type="password" autocomplete="off" placeholder="CVC"  oninput="comNpay.checkAndcreateToken()" class="classCardPapam">
        		</p>
          <input type="hidden" id="token" name="token" required/>
        		<div class="clear"></div>';

			do_action( 'woocommerce_credit_card_form_end', $this->id );

			echo '<div class="clear"></div></fieldset>';

		}


		/*
		 * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
		 */
		public function payment_scripts() {


			// we need JavaScript to process a token only on cart/checkout pages, right?
			if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
				return;
			}

			// if our payment gateway is disabled, we do not have to enqueue JS too
			if ( 'no' === $this->enabled ) {
				return;
			}

			// no reason to enqueue JavaScript if API keys are not set
			if ( empty( $this->private_key ) || empty( $this->publishable_key ) ) {
				return;
			}

			// do not work with card detailes without SSL unless your website is in a test mode
			if ( ! $this->testmode && ! is_ssl() ) {
				return;
			}

			wp_enqueue_script( 'papam_js', plugin_dir_url( __FILE__ ).'papamsdk.js' );

			wp_register_script( 'woocommerce_papam', plugins_url( 'papam.js', __FILE__ ), array( 'jquery', 'papam_js' ) );

			wp_localize_script( 'woocommerce_papam', 'papam_params', array(
				'publishableKey' => $this->publishable_key,
				'apiurl' => $this->url,
				'tokenregurl' => plugin_dir_url( __FILE__ ).'tokenreg.php',
				'temptokenoption' => 'temptoken_'.get_current_user_id(),
				'origin' => $_SERVER['HTTP_HOST'],
			) );

			wp_enqueue_script( 'woocommerce_papam' );

		}

		/*
		 * We're processing the payments here
		 */
		public function process_payment( $order_id )
		{
			global $woocommerce;

			$order = new WC_Order($order_id);


			if (!session_id()) {
				session_start();
			}

			$transactionRef = $order->get_order_key() . date('is');
			/*
            * Array with parameters for API interaction
            * /rest/payment/tokenDebit
            */

            $browserInfos = array("browserJavascriptEnabled" => false);
            if (isset($_SESSION['browser'])) {
                $browserInfos = $_SESSION['browser'];
            }

			unset($args);
			$args = array(
				"serialNumber" => $this->publishable_key,
				"key" => $this->private_key,
				"tokenRef" => $_SESSION['papamtoken'],
				"transactionRef" => $transactionRef,
				"amount" => round($order->get_total('total', true), 2) * 100,
				"force3ds" => 0,
				"redirection3DSV2Url" => plugin_dir_url(__FILE__)."3dsback.php?order_id=".$order_id."&transaction_ref=".$transactionRef,
				"browser" => json_encode($browserInfos),
				"customer" => json_encode(array
				(
					"firstName" => $order->get_billing_first_name(),
					"lastName" => $order->get_billing_last_name(),
					"email" => $order->get_billing_email(),
					"road" => $order->get_billing_address_1(),
					"zipCode" => $order->get_billing_postcode(),
					"city" => $order->get_billing_city(),
					"country" => $order->get_billing_country(),
					"phone" => $order->get_billing_phone(),
				)),
			);

			if(!$_SESSION['papamtoken']){
				wc_add_notice(__('Error, please verify card informations ', 'papam'), 'error');
				return;
			}
			unset($_SESSION['papamtoken']);


			$response = wp_remote_post($this->debiturl, array(
				'method' => 'POST',
				'timeout' => 50,
				'blocking' => true,
				'sslverify' => false,
				'body' => $args,
			));


			if (!is_wp_error($response)) {
				$body = json_decode($response['body'], true);


				if ($body['ok'] == 1) {
					// 3ds check
					if ($body['actionCode'] == 'AUTH_3DS_REQUISE') {

						$order->update_status('on-hold', __('Awaiting 3ds payment', 'woocommerce'));

                        if ($body['transaction']["verifyEnrollment3dsV1"]) {
                            // 3DS V1
                            $acs_url = rawurlencode($body['transaction']['verifyEnrollment3dsActionUrl']);
                            $authencity_token = rawurlencode($body['transaction']["verifyEnrollment3dsMd"]);
                            $pareq = rawurlencode($body['transaction']["verifyEnrollment3dsPareq"]);
                            $term_url = rawurlencode(plugin_dir_url(__FILE__) . "3dsback.php?order_id=".$order_id."&transaction_ref=".$transactionRef);
                            $url = plugin_dir_url(__FILE__) . "3dsform.php?acs_url=".$acs_url."&pareq=".$pareq."&authencity_token=".$authencity_token."&term_url=".$term_url;
                        }
                        else {
                            // 3DS V2
                            $acs_url = rawurlencode($body['transaction']['verifyEnrollment3dsActionUrl']);
                            $creq = rawurlencode($body['transaction']["verifyEnrollment3dsCreq"]);
                            $url = plugin_dir_url(__FILE__) . "3dsform.php?acs_url=".$acs_url."&creq=".$creq."&term_url=".$term_url;
                        }

						return array(
							'result' => 'success',
							'redirect' => $url
						);

					} else {

						// we received the payment
						$order->payment_complete();
						$order->reduce_order_stock();


						// Empty cart
						$woocommerce->cart->empty_cart();

						// Redirect to the thank you page
						return array(
							'result' => 'success',
							'redirect' => $this->get_return_url($order)
						);
					}
				} else {
					//wc_add_notice(  'Please try again. '.$body['message'], 'error' );
					wc_add_notice(__('Error, Transaction has been refused. Contact your bank or the merchant', 'papam'), 'error');
					return;
				}
			} else {
				$error_message = $response->get_error_message();
				wc_add_notice('Connection error. ' . $error_message, 'error');
				return;
			}

			session_write_close();
		}

	}
}